package Prism.core;

import java.util.Vector;

import org.apache.log4j.Logger;

import Prism.exception.ComponentStateException;
import Prism.extensions.port.*;

/**
 * This class defines the basic Prism component and the send methods that
 * allow any class extending this one to interact with the architecture.
 * The usual practice is to create a component in your own architecture
 * by extending this class and implementing the handle method.
 *
 *@version 2.0
 *@author USC Soft. Arch. Group. Contact: Sam Malek <A HREF="mailto:malek@usc.edu"> malek@usc.edu </A>
 */
public class Component extends Brick implements IComponent, java.io.Serializable 
{
    protected transient Vector ports;
    protected AbstractImplementation implementation;
    protected int activeThreads = 0;
    protected boolean passivating = false;
    protected Logger logger;
    
	protected float usedTime = 0;
	protected long startTime = 0;
	protected long previousStart = 0;
       
  /**
    * This constructor calls on the <code>Brick(String)</code> constructor.
    *
    * @param name	String: the name of this component object. 
    */
   public Component (String name)
   {
	   super(name);
	   ports=new Vector(CONNS_PER_COMPONENT);
	   style = PrismConstants.DEFAULT;
	   startTime = System.currentTimeMillis();
   }
    
   public Component (String name, AbstractImplementation pImplementation)
   {
	   super(name);	   
	   ports=new Vector(CONNS_PER_COMPONENT);
	   style = PrismConstants.DEFAULT;
	   implementation = pImplementation;
	   implementation.setAssociatedComponent(this);
	   startTime = System.currentTimeMillis();
   }
   
   public Component (String name, AbstractImplementation pImplementation, Logger log) {
	   super(name);	   
	   ports=new Vector(CONNS_PER_COMPONENT);
	   style = PrismConstants.DEFAULT;
	   implementation = pImplementation;
	   implementation.setAssociatedComponent(this);	   
	   startTime = System.currentTimeMillis();
	   this.logger = log;
   }

   public void start()
   {
   		if (implementation != null)
   			implementation.start();
   }
   
   public void setImplementation (AbstractImplementation impl)
   {
   		implementation = impl;
   }
   
   public AbstractImplementation getImplementation ()
   {
   		return implementation;
   }
   
  /**
    * This method sends the event up/down the Prism architecture that this component is a
    * part of.
    *
    * @param e		a Event to be sent to the Brick above/below.
    */   
   public void send(Event e)
   {
	   log("Sending AT: " + name + " Event: " + e.name + " >> Names: " + e.parameterName + " Values: " + e.parameterValue + " Statues: " + status);
	   	
//	   while(getStatus() == PrismConstants.PASSIVE) {
//		   //Assuming the middleware has already passivated the components that may send request to this component.
//		   //Therefore this event is not generated during procesing an incomming transaction.
//		   //This means that this while loop only delays the component from initiating new transactions until the component reactivated.
//		   try {
//				this.wait();
//			} catch (InterruptedException e1) {
//				e1.printStackTrace();
//			}		   
//	   }
	   
	   //It is active or just has been inactivated		   
		for (int i=0; i < ports.size(); i++)
		{                
		    IPort thisPort = (IPort)ports.elementAt(i);
		    if ( thisPort.getPortType() == e.eventType )
		        {
		            if (thisPort instanceof ExtensiblePort)
		            {
		                Event e1 = e.replicate();
		                e1.handlingBrick = (Brick)thisPort;
		                add(e1);
		            }
		            else if (thisPort instanceof Port)
		            {
		                Event e1 = e.replicate();
		                IPort mutualPort = thisPort.getMutualPort();
		                e1.handlingBrick = mutualPort.getParentBrick();
		                add(e1);
		            }                        
		        }
		}
   }

  /**
    * Handling of Event. This is application specific code and should be implemented by subclasses of this class.
    *
    * @param e	Event to be handled
    */   
   public void handle(Event e)
   {
	   synchronized (this) {
		   log("Handling AT: " + name + " Event: " + e.name + " >> Names: " + e.parameterName + " Values: " + e.parameterValue + " Statues: " + status);
		   	
		   while(getStatus() == PrismConstants.PASSIVE) {
			   //TODO send it back to the Prism Queue
			   //Assuming the middleware has already passivated the components that may send request to this component.
			   //Therefore this event is generated while the middleware trying to reactiavet those components.
			   //This means that this while loop only delays the component from providing service until middleware finishes with the layer below and reactivate this component.
			   try {
					this.wait();
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}		   
		   }
	   }

	   //It is active or inactive
	   if (implementation != null)
			implementation.handle(e);
   }

   /**
    * Adds a port to this component. 
    * @param port    Port to be added.
    */
   public void addCompPort(IPort port) 
   {
   		port.setParentBrick(this);
		if (ports == null)
		{
		   	ports=new Vector(CONNS_PER_COMPONENT);
	       	ports.addElement(port);       	
		}
		else
		   	ports.addElement(port);
   }   

   /**
    * Removes a port from this component.
    *@param port    Port to be removed.
    */
   public void removeCompPort(IPort port) 
   {
   		if (ports == null)
   			return;
        else
        {	
       		ports.removeElement(port);
       		port.setParentBrick(null);
        }
   }  
   
   /**
    * This method returns a list of all available ports in this component.
    *@return DynamicArray   List of available ports  
    */
   public Vector getCompPorts()
   {
       	return ports;
   }

   public int getStatus() {
	   return status;
   }
   
	synchronized public boolean activate() {
		if(status == PrismConstants.PASSIVE)
			return false;
		
		activeThreads++;
		
		if(activeThreads == 1) {
			status = PrismConstants.ACTIVE;
			previousStart = System.currentTimeMillis();
			log(name + " is activate.");
		}
		
		return true;
	}
	
	synchronized public void inactivate() {
		if(status != PrismConstants.ACTIVE || activeThreads <= 0)
			throw new ComponentStateException("The component '" + name + "' should be active to be inactivated.");
		
		activeThreads--;
		
		if(activeThreads == 0) {
			status = PrismConstants.INACTIVE;
			usedTime += (System.currentTimeMillis() - previousStart);
			previousStart = 0;
			log(name + " is inactivate.");
			notifyAll();
		}
	}
	
	synchronized public boolean passivate() {
		if(status == PrismConstants.ACTIVE)
			return false;
		
		status = PrismConstants.PASSIVE;
		passivating = false;
		log(name + " is passive.");
		this.notifyAll();
		
		return true;
	}   

	synchronized public void reactivate() {
		if(status == PrismConstants.PASSIVE) {
			status = PrismConstants.INACTIVE;
			log(name + " is inactivate.");
			this.notifyAll();
		}
	}
   
	synchronized public void setPassivating() {
	   passivating = true;
	}
	
	public void log(String text) {
		logger.trace(text);
	}
	
	public void info(String text) {
		logger.info(text);
	}
	
	public void setLog(Logger log) {
		this.logger = log;
	}

	public boolean isPassivating() {
		return passivating;
	}

	public String getName() {
		return name;
	}
	
	public float getUsageRatio() {		
		float total = System.currentTimeMillis() - startTime;
		
		if(previousStart == 0)
			return usedTime / total;
		else
			return (usedTime + (System.currentTimeMillis() - previousStart)) / total;
	}		
}